#! /usr/bin/env python
# flake8: noqa E741


def dameraulevenshtein(seq1, seq2):
    """Calculate the Damerau-Levenshtein distance between sequences.

    This distance is the number of additions, deletions, substitutions,
    and transpositions needed to transform the first sequence into the
    second. Although generally used with strings, any sequences of
    comparable objects will work.

    Transpositions are exchanges of *consecutive* characters; all other
    operations are self-explanatory.

    This implementation is O(N*M) time and O(M) space, for N and M the
    lengths of the two sequences.
    """
    # codesnippet: D0DE4716-B6E6-4161-9219-2903BF8F547F
    # Conceptually, this is based on a len(seq1) + 1 * len(seq2) + 1 matrix.
    # However, only the current and two previous rows are needed at once,
    # so we only store those.
    oneago = None
    thisrow = range(0, len(seq2) + 1)
    for x in range(len(seq1)):
        # Python lists wrap around for negative indices, so put the
        # leftmost column at the *end* of the list. This matches with
        # the zero-indexed strings and saves extra calculation.
        twoago, oneago, thisrow = oneago, thisrow, [0] * len(seq2) + [x + 1]
        for y in range(len(seq2)):
            delcost = oneago[y] + 1
            addcost = thisrow[y - 1] + 1
            subcost = oneago[y - 1] + (seq1[x] != seq2[y])
            thisrow[y] = min(delcost, addcost, subcost)
            # This block deals with transpositions
            if x > 0 and y > 0 and seq1[x] == seq2[y - 1] and seq1[x - 1] == seq2[y] and seq1[x] != seq2[y]:
                thisrow[y] = min(thisrow[y], twoago[y - 2] + 1)
    return thisrow[len(seq2) - 1]


def min_dist(s1, d, seuil):
    # donne le nom de la filiere a partir d'une chaine approchant....
    # d2 = {"hydraulique": "HY", "hy": "HY", "hydro": "HY", "electronique": "EN", "en": "EN", "electronique": "EN",
    #      "genie electrique et automatique": "GEA", "gea": "GEA", "genie electrique": "GEA", "ima": "IMA",
    #      "info": "IMA", "informatique": "IMA", "in":  "IMA", "tr": "TR", "telecom": "TR", "reseau": "TR"}
    seuil_actuel = seuil
    mot_actuel = ""

    for i in d.keys():
        seuil_temp = dameraulevenshtein(s1.lower(), i) / float(max(len(s1), len(i)))
        if seuil_temp < seuil_actuel:
            mot_actuel = i
            seuil_actuel = seuil_temp
    if seuil_actuel < seuil:
        print(d[mot_actuel])
        return d[mot_actuel]
    else:
        return 0


def trouve_combi(l, final):
    d2 = {"3a": 3, "2a": 2, "1a": 1, "3": 3, "2": 2, "1": 1}
    d3 = {
        "hydraulique": 1,
        "hy": 1,
        "hydro": 1,
        "en": 2,
        "electronique": 2,
        "genie electrique et automatique": 3,
        "gea": 3,
        "genie electrique": 3,
        "ima": 4,
        "info": 4,
        "informatique": 4,
        "in": 4,
        "tr": 5,
        "telecom": 5,
        "reseau": 5,
        "telecom et reseau": 5,
        "telecom reseau": 5,
    }
    for s in l:
        s_temp = s[0:2].lower()
        if s_temp in d2:
            s_temp2 = s[2 : len(s)]
            ind = min_dist(s_temp2, d3, 0.5)
            if ind != 0:
                final[0].append(ind)
                final[1].append(d2[s_temp])
                l.remove(s)
                return
        s_temp = s[0:1].lower()
        if s_temp in d2:
            s_temp2 = s[1 : len(s)]
            ind = min_dist(s_temp2, d3, 0.5)
            if ind != 0:
                final[0].append(ind)
                final[1].append(d2[s_temp])
                l.remove(s)


def cut_string(s1):
    # fonction de decoupage de la chaine s1 selon les espaces
    # si on entre "tomtom et nana", elle renvoit ['tomtom', 'et', 'nana']
    # elle fait de la merde s'il y plus d'un espace entre chaque mot et s'il y a des espaces avant et apres la chaine
    # a corriger!!!!!
    l = []
    indice_debut = 0
    for i in range(0, len(s1), 1):
        if s1[i] == " ":
            l.append(s1[indice_debut:i])
            indice_debut = i
    l.append(s1[indice_debut + 1 : len(s1)])
    # print(l)
    return l


def list_of_string_to_string(l, indice_debut, indice_fin):
    s = ""
    for i in range(indice_debut, indice_fin, 1):
        s = s + " " + l[i]
    if s[0] == " ":
        s = s[2:]
    return s


def l2s(l, i, j):
    return list_of_string_to_string(l, i, j)


# elimine les doublons dans la liste triee simplement (par sort())
def suppr_doublons(l):
    final = []
    if len(l) == 0:
        return final
    if len(l) == 1:
        final.append(l[0])
        return final
    else:
        final.append(l[0])
        for i in range(1, len(l)):
            if l[i] != l[i - 1]:
                final.append(l[i])
        return final


def trouve_filiere(l):
    d2 = {
        "hydraulique": 1,
        "hy": 1,
        "hydro": 1,
        "en": 2,
        "electronique": 2,
        "genie electrique et automatique": 3,
        "gea": 3,
        "genie electrique": 3,
        "ima": 4,
        "info": 4,
        "informatique": 4,
        "in": 4,
        "tr": 5,
        "telecom": 5,
        "reseau": 5,
        "telecom et reseau": 5,
        "telecom reseau": 5,
    }
    indice_debut = 0
    indice_fin = 0
    final = []
    while indice_fin < len(l):
        ind = min_dist(" ".join(l[indice_debut : indice_fin + 1]), d2, 0.5)
        if ind != 0:
            final.append(ind)
            del l[indice_debut : indice_fin + 1]
            # indice_debut = indice_fin + 1
            indice_fin = indice_debut

        else:
            indice_fin = indice_fin + 1
        if indice_debut < indice_fin and indice_fin >= len(l):
            # print("mot perdu")
            indice_debut = indice_debut + 1
            indice_fin = indice_debut
    final.sort()
    final = suppr_doublons(final)
    return final


def trouve_annee_en_cours(l):
    d2 = {"3a": 3, "2a": 2, "1a": 1, "3": 3, "2": 2, "1": 1}
    indice_debut = 0
    indice_fin = 0
    final = []
    while indice_fin < len(l):
        ind = min_dist(" ".join(l[indice_debut : indice_fin + 1]), d2, 0.5)
        if ind != 0:
            final.append(ind)
            del l[indice_debut : indice_fin + 1]
            # indice_debut = indice_fin + 1
            indice_fin = indice_debut

        else:
            indice_fin = indice_fin + 1
        if indice_debut < indice_fin and indice_fin >= len(l):
            # ("mot perdu")
            indice_debut = indice_debut + 1
            indice_fin = indice_debut
    final.sort()
    final = suppr_doublons(final)
    return final


def construire_requete(l):
    requete = [[], [], [], []]
    trouve_combi(l, requete)
    requete[0].extend(trouve_filiere(l))
    requete[1].extend(trouve_annee_en_cours(l))
    requete[2].extend(l)
    for i in range(len(requete)):
        requete[i].sort()
        requete[i] = suppr_doublons(requete[i])
    return requete
